// Drew Waranis

import org.json.simple.*;
import org.json.simple.JSONObject;
import org.json.simple.JSONArray;
//import org.json.simple.parser.*;

import net.glxn.qrgen.*;
import net.glxn.qrgen.image.*;
import net.glxn.qrgen.exception.*;
import java.io.ByteArrayOutputStream;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.awt.Toolkit;
import java.awt.Image;


int cols = 4, rows = 3;

Grate grate;

int numAlbums = 10;

Album[] pics = new Album[numAlbums*cols*rows];

boolean bill = false;

int twiddle = 60;

JSONObject images;

void setup() {
  size(1280, 1024);

  String jsonString = join(loadStrings("combined.json"), "\n");
  JSONObject obj = (JSONObject)( JSONValue.parse(jsonString) );
  JSONArray data = (JSONArray)( obj.get("data") );

  for (int i = 0; i < 12; i++){
    JSONArray m = (JSONArray)( data.get(i) );

    for (int j = 0; j < 10; j++){
          JSONObject a = (JSONObject) m.get(j);

          JSONObject b = (JSONObject) m.get(0);
          int first = Integer.parseInt( (String) b.get("count").toString() );

          pics[10*i + j] = new Album(a,first);
    }    
  }

  grate = new Grate(cols, rows);

  smooth();
}

void draw() {
  background(0);

  grate.push();

  for (int i = 0; i < cols*rows; i++){
    grate.pushWindow(i);
    month_layout(i);
    grate.popWindow();
  }

  grate.pop();
}

void keyPressed() {
  if (key == CODED) {
    if (keyCode == LEFT) {
      grate.moveLeft();
    } else if (keyCode == RIGHT) {
      grate.moveRight();
    } else if (keyCode == DOWN) {
      if (grate.overview)
        grate.moveDown();
    } else if (keyCode == UP) {
      if (grate.overview)
        grate.moveUp();
    }
  } else {
    if (key == ' ') {
      grate.overview = !grate.overview;
    } else if (key == 'b') {
      for (int i = 0; i<pics.length; i++){
        pics[i].var = !pics[i].var;
      }
    }
  }
}

void mousePressed() {
  for (int i = 0; i < numAlbums*cols*rows; i++){
    //int pos = grate.displayNum()*numAlbums + i;
    int low = grate.displayNum()*numAlbums;
    int high = (grate.displayNum()+1)*numAlbums;

    if ( mouseX > pics[i].x && mouseX < pics[i].x + pics[i].aDim && mouseY > pics[i].y && mouseY < pics[i].y + pics[i].aDim && low <= i && i < high ){
      pics[i].dispDetails = true;
    } else {
      pics[i].dispDetails = false;
    }
  }
}

class Grate {

  int rows, cols, diff, fcols, frows;
  int xpos = 0, ypos = 0;

  float xtemp, ytemp;

  boolean overview = false;

  Grate(int side){
    rows = cols = side;
    diff = 0;
    frows = fcols = side;
    xtemp = ytemp = 0;
  }

  Grate(int tcols, int trows){
    rows = frows = trows;
    cols = fcols = tcols;
    diff = cols - rows;
    xtemp = 0;
    ytemp = 0;
  }

  void push(){
    if (overview){
      displayOverview();
    } else {
      displayWindow();
    }
  }

  void pop(){
    if (overview){
      stroke(255/4);
      strokeWeight(3);
      noFill();
      rect(xpos*width, ypos*height, width, height);
    }
  }

  void pushWindow(int win){
    pushMatrix();

    int i = win % cols;
    int j = floor(win / cols);
    translate(i*width, j*height);
  }

  void popWindow(){
    popMatrix();
  }

  void displayWindow(){
    translate(xpos*(-width), ypos*(-height));
  }

  void displayOverview(){
    int fcols = cols, frows = rows;

     textAlign(CENTER, CENTER);
     textSize(32);
     text("Top 10 Monthly Album Checkouts From SPL In 2012", width/2, height/(frows*4));

     textAlign(RIGHT, TOP);
     textSize(16);
     text("By Drew@Waran.is", width, 0);

    if (diff > 0){
      frows = rows + diff;
      translate(0., height/(frows*2));
    } else if (diff < 0) {
      fcols = cols + abs(diff);
      translate(width/(fcols*2), 0.);
    }

    scale(1./fcols, 1./frows);

    // stroke(255);
    // strokeWeight(1);
    // for (i = 0; i <= rows; i++){
    //   line(0., i*height, cols*width, i*height);
    // }
    // for (i = 0; i <= cols; i++){
    //   //line(i*width, 0., i*width, (rows-1)*height);
    //   line(i*width, 0., i*width, rows*height);
    // }
  }

  int displayNum(){
    return ypos*cols + xpos;
  }

  void moveLeft(){
    if (xpos > 0){
      xpos--;
    } else if (ypos == 0) {
      if (overview){
        xpos = cols - 1;
        ypos = rows - 1;
      }
    } else {
      xpos = cols - 1;
      moveUp();
    }

  }
  void moveRight(){
    if (xpos < cols - 1) {
      xpos++;
    } else if (ypos == rows - 1) {
      if (overview){
        xpos = 0;
        ypos = 0;
      }
    } else {
      xpos = 0;
      moveDown();
    }

  }
  void moveDown(){
    ytemp = ypos;

    if (ypos < rows - 1)
      ypos++;
  }
  void moveUp(){
    ytemp = ypos;

    if (ypos > 0)
      ypos--;
  }
};


// add grids
class Album {
  PImage aImage, aHalf, aAlt, qr;
  int aDim;
  int altDim;
  boolean var = false;
  String artist, title, url;
  boolean dispDetails = false;
  float x = 0, y = 0;
  float offx, offy;

  Album(JSONObject details, int first){
    aDim = width/5 * Integer.parseInt( (String) details.get("count").toString() ) / first;
    altDim = aDim*3/4;

    aImage = loadImage("data/img/" + details.get("title")+"-"+details.get("artist")+".jpg");
    aImage.resize(aDim, 0);

    aHalf = createImage(aDim/2, aDim, RGB);
    aHalf.copy(aImage, 0, 0, aDim/2, aDim, 0, 0, aDim/2, aDim);

    aAlt = createImage(aDim, altDim, RGB);
    aAlt.copy(aImage, 0, 0, aDim, aDim, 0, 0, aDim, altDim);

    artist = (String) details.get("artist");
    title = (String) details.get("title");
    url = (String) details.get("url");

    qr = linkToQR(url, aDim);
  }

  void display2d(float ix, float iy){
    x = ix;
    y = iy;

    stroke(255);
    strokeWeight(.5);
    noFill();
    if (var){
      image(aAlt, x, y);
      image(aHalf, x, y);

      line(x, y, x+aDim, y);
      line(x+aDim, y, x+aDim, y+altDim);
      line(x+aDim, y+altDim, x + aDim/2, y+altDim);
      line(x + aDim/2, y+altDim, x + aDim/2, y+aDim);
      line(x + aDim/2, y+aDim, x, y+aDim);
      line(x, y+aDim, x, y);
    } else {
      image(aImage, x, y);
      rect(x, y, aDim, aDim);
    }

    if (dispDetails){
      image(qr, x, y);

      float buff = 0;

      textSize(16);
      float twide = textWidth(artist) > textWidth(title) ? textWidth(artist) : textWidth(title);

      if (aDim > twide){
        twide = aDim;
      } else {
        buff = 10;
      } 

      if (x + aDim/2 - twide/2 - buff < 0){
        fill(255/8);
        rect(0, y + aDim, twide + 2*buff, twiddle, /**/ 0, 0, aDim/8, 0);

        textAlign(LEFT, TOP);
        fill(255);
        text(title + "\n" + artist, 10, y + aDim + 10);
      } else if (x + aDim/2 + twide/2 + buff > width) {
        fill(255/8);
        rect(width - twide - buff*2, y + aDim, twide + 2*buff, twiddle, /**/ 0, 0, 0, aDim/8);

        textAlign(RIGHT, TOP);
        fill(255);
        text(title + "\n" + artist, width - 10, y + aDim + 10);
      } else {
        fill(255/8);
        rect(x + aDim/2 - twide/2 - buff/2, y + aDim, twide + buff, twiddle, /**/ 0, 0, aDim/8, aDim/8);

        textAlign(CENTER, TOP);
        fill(255);
        text(title + "\n" + artist, x + aDim/2, y + aDim + 10);
      }
      
    }
  }

  void offset(float offx, float offy){
    x += offx;
    y += offy;
  }
};

// Modified http://forum.processing.org/topic/converting-bufferedimage-to-pimage#25080000000340208 .
// Generates a sized QR code of a passed URL.
public PImage linkToQR(String url, int dim) {
  try {
    byte[] m = QRCode.from(url).to(ImageType.PNG).withSize(dim,dim).stream().toByteArray();
    Image im = Toolkit.getDefaultToolkit().createImage(m);
    return loadImageMT(im);
  }
  catch(Exception e) {
    System.err.println("Can't create image from buffer");
    e.printStackTrace();
  }
  return null;
}
