#!/usr/bin/env ruby

# Required rubygems.
require 'json'
require 'bitly'
require 'vacuum'
require 'xmlsimple'
require 'open-uri'

# Uses https://github.com/philnash/bitly to shorten URLs for QR codes.
# Function takes the URL to be shortened.
# To-do: use API v3 before v2 is removed.
def bitly(url)
  # Insert your own username and key 
  bitly = Bitly.new("drewrwx", "R_dfecaa0d8eee498e78cefa8d2afd3ade")
  u = bitly.shorten(url)

  return u.short_url
end


# Uses https://github.com/hakanensari/vacuum to get album details from Amazon's API.
def ama(artist, album)
  req = Vacuum.new

  # Insert your own values here.
  tag = "stt01-20"
  req.configure key:    'AKIAIZXITNB3UWXEGDVQ',
                secret: 'QrZGaBJjAD0WvR58QoRqdm26QuMiaN9T78fWzRkZ',
                tag:    tag

  # Gets a "small" subset of album info and image links.
  scopes = %w(
    Small Images
  ).join ','

  # Searches albums using the album name and artist.
  params = {
    'Operation'     => 'ItemSearch',
    'Artist'        => artist,
    'Title'         => album,
    'ResponseGroup' => scopes,
    'SearchIndex'   => 'Music'
  }

  # Get the results and convert them to a ruby hash.
  res = req.get query: params
  data = XmlSimple.xml_in(res.body)

  # Returns blank values if album isn't found.
  # I replaced these values by hand.
  if data["Items"][0]["Item"].to_s == ''
    return ["","","","","",""]
  end

  # Use these variables to shorthand the return statement.
  item = data["Items"][0]["Item"][0]
  attrib = item["ItemAttributes"][0]
  large = item["LargeImage"][0]

  # Uncomment to dump album info into the 'albums' directory.
  # File.open("data/albums/#{attrib["title"][0]}-#{attrib["Artist"][0]}.json","w") do |f|
  #   f.write( JSON.pretty_generate(JSON.parse(data.to_json)))
  # end

  # Returns array of [artist, title, short_url, image_link] for each album.
  return [
           attrib["Artist"][0],
           attrib["Title"][0],
           bitly("http://www.amazon.com/dp/"+item["ASIN"][0]+"?tag=#{tag}"),
           large["URL"][0]
         ]
end

# Uses the previous two functions to add/correct info in the SPL queries.
# Combines the monthly SPL JSON into a master JSON.
def process()
  # Creates 'img' directory, otherwise aborts function.
  if (!File.directory?("data/img"))
    FileUtils.mkdir_p("data/img")
  else
    puts "Script already run.\n"
    return
  end

  # Creates hash and adds entry to format like JSON.
  comb = Hash.new
  comb["data"] = []

  # Create hash to download images with.
  # A large number of albums appear multiple times,
    # so thsi cuts down on downloads from Amazon. 
  images = Hash.new

  (0 .. 11).each do |i|
    # Copy each month's data into an array entry.
    comb["data"][i] = JSON.parse( IO.read("data/source/#{i+1}-2012.json") )["data"]

    (0 .. 9).each do |j|
      # Lookup each album on Amazon and replace details with proper results.
      out = ama(comb["data"][i][j]["artist"], comb["data"][i][j]["title"])
      puts out.inspect

      comb["data"][i][j]["artist"] = out[0]
      comb["data"][i][j]["title"]  = out[1]
      comb["data"][i][j]["url"]    = out[2]

      # Store the image url in the images hash.
      key = comb["data"][i][j]["title"] + "-" + comb["data"][i][j]["artist"]
      images[key] = out[3]

      # Rate limit so that I can do this in one sweep.
      sleep(1)
    end
  end

  # Save the master JSON file.
  File.open("data/combined.json","w") do |f|
    f.write( JSON.pretty_generate(JSON.parse(comb.to_json)))
  end

  # Download album art to 'data/img' using images hash.
  images.each_pair {|key, url|
    File.open("data/img/#{key}.jpg", "wb") do |saved_file|
      # The following "open" is provided by open-uri.
      open(url, 'rb') do |read_file|
        saved_file.write(read_file.read)
      end
    end
  }
end

process